
 ********************************************************************/
#ifndef USB_H
#define USB_H

/*
 * usb.h provides a centralize way to include all files
 * required by Microchip USB Firmware.
 *
 * The order of inclusion is important.
 * Dependency conflicts are resolved by the correct ordering.
 */

#include "typedefs.h"
#include "usb_config.h"
#include "usb_device.h"
#include "HardwareProfile.h"

#if defined(USB_USE_HID)                // See usb_config.h
#include "usb_device_hid.h"
#endif


//These callback functions belong in your main.c (or equivalent) file.  The USB
//stack will call these callback functions in response to specific USB bus events,
//such as entry into USB suspend mode, exit from USB suspend mode, and upon
//receiving the "set configuration" control tranfer request, which marks the end
//of the USB enumeration sequence and the start of normal application run mode (and
//where application related variables and endpoints may need to get (re)-initialized.
void USBCBSuspend(void);
void USBCBWakeFromSuspend(void);
void USBCBInitEP(uint8_t ConfigurationIndex);
void USBCBCheckOtherReq(void);


//API renaming wrapper functions
#define HIDTxHandleBusy(a)   {mHIDTxIsBusy()}
#define HIDRxHandleBusy(a)   {mHIDRxIsBusy()}

#endif //USB_H
