/********************************************************************
 *******************************************************************/

#include <p24FJ256GB110.h>
#include <system.h>
#include <system_config.h>
#include <usb/usb.h>
#include <leds.h>
#include <buttons.h>
#include <adc.h>

/** CONFIGURATION Bits **********************************************/
_CONFIG1(
    JTAGEN_OFF      &
    GCP_OFF         &
    GWRP_OFF        &
    FWDTEN_OFF      &
    ICS_PGx2
);

_CONFIG2( 
    PLL_96MHZ_ON    &
    IESO_OFF        &
    FCKSM_CSDCMD    &
    OSCIOFNC_ON     &
    POSCMOD_HS      &
    FNOSC_PRIPLL    &
    PLLDIV_DIV2     &
    IOL1WAY_ON
);
 
/*********************************************************************
* Function: void SYSTEM_Initialize( SYSTEM_STATE state )
*
* Overview: Initializes the system.
*
* PreCondition: None
*
* Input:  SYSTEM_STATE - the state to initialize the system into
*
* Output: None
*
********************************************************************/
void SYSTEM_Initialize( SYSTEM_STATE state )
{
    switch(state)
    {
        case SYSTEM_STATE_USB_START:
            //Switch to alternate interrupt vector table for bootloader
            INTCON2bits.ALTIVT = 1;
            BUTTON_Enable(BUTTON_USB_DEVICE_HID_CUSTOM);

            if((BUTTON_IsPressed(BUTTON_USB_DEVICE_HID_CUSTOM)==false) && ((RCON & 0x83) != 0))
            {
                //Switch to app standare IVT for non boot mode
                INTCON2bits.ALTIVT = 0;
                __asm__("goto 0x1800");
            }
            
            LED_Enable(LED_USB_DEVICE_STATE);
            LED_Enable(LED_USB_DEVICE_HID_CUSTOM);
            
            break;
            
        case SYSTEM_STATE_USB_SUSPEND:
            break;
            
        case SYSTEM_STATE_USB_RESUME:
            break;
    }
}

#if defined(USB_INTERRUPT)
void __attribute__((interrupt,auto_psv)) _AltUSB1Interrupt()
{
    USBDeviceTasks();
}
#endif


        
