/********************************************************************
 *******************************************************************/

#ifndef USART_H
#define USART_H

#include <stdbool.h>

#define CLOCK_FREQ 64000000
#define GetSystemClock() CLOCK_FREQ

#define UART_ENABLE   RCSTAbits.SPEN


#define UART_TRISTx   TRISCbits.TRISC6
#define UART_TRISRx   TRISCbits.TRISC7
#define UART_Tx       PORTCbits.RC6
#define UART_Rx       PORTCbits.RC7

// Use following only for Hardware Flow Control
#define UART_DTS PORTBbits.RB0
#define UART_DTR LATDbits.LATD7
#define UART_RTS LATBbits.LATB2
#define UART_CTS PORTBbits.RB1

#define mInitRTSPin() {TRISBbits.TRISB2 = 0;}   //Configure RTS as a digital output.
#define mInitCTSPin() {TRISBbits.TRISB1 = 1;}   //Configure CTS as a digital input.  (Make sure pin is digital if ANxx functions is present on the pin)
#define mInitDTSPin() {TRISBbits.TRISB0 = 1;}   //Configure DTS as a digital input.  (Make sure pin is digital if ANxx functions is present on the pin)
#define mInitDTRPin() {TRISDbits.TRISD7 = 0;}   //Configure DTR as a digital output.

/** Register/Bit Aliases ********************************************/
    #define TXREG         TXREG1
    #define RCREG         RCREG1
    //#define RCSTA         RCSTA1
    //#define RCSTAbits     RCSTA1bits
    //#define TXSTA         TXSTA1
    //#define TXSTAbits     TXSTA1bits
    #define SPBRG         SPBRG1
    #define SPBRGH        SPBRGH1
    #define BAUDCON       BAUDCON1
    #define RCIF          RC1IF


//typedef unsigned char           BYTE;                      /* 8-bit unsigned  */
//typedef unsigned long           DWORD;                     /* 32-bit unsigned */
//
//typedef union
//{
//    DWORD Val;
//    unsigned char v[4];
//
//    struct
//    {
//        BYTE LB;
//        BYTE HB;
//        BYTE UB;
//        BYTE MB;
//    } byte;
//}DWORD_VAL;

/*********************************************************************
* Function: void USART_Initialize(void);
*
* Overview: Initializes USART (RS-232 port)
*
* PreCondition: None
*
* Input: None
*
* Output: None
*
********************************************************************/
void USART_Initialize();

/******************************************************************************
 * Function:        void USART_putcUSART(char c)
 *
 * PreCondition:    None
 *
 * Input:           char c - character to print to the UART
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Print the input character to the UART
 *
 * Note:
 *
 *****************************************************************************/
void USART_putcUSART(char);

/******************************************************************************
 * Function:        void USART_mySetLineCodingHandler(void)
 *
 * PreCondition:    USB_CDC_SET_LINE_CODING_HANDLER mySetLineCodingHandler is defined
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function gets called when a SetLineCoding command
 *                  is sent on the bus.  This function will evaluate the request
 *                  and determine if the application should update the baudrate
 *                  or not.
 *
 * Note:
 *
 *****************************************************************************/
void USART_mySetLineCodingHandler(void);

/******************************************************************************
 * Function:        unsigned char USART_getcUSART()
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          unsigned char c - character to received on the UART
 *
 * Side Effects:    None
 *
 * Overview:        Get the input character from the UART
 *
 * Note:
 *
 *****************************************************************************/
unsigned char USART_getcUSART(void);

#endif //USART_H
